@@ -17,9 +17,11 @@
 
 
 
-if(getRversion() >= "2.15.1")  {
+if (getRversion() >= "2.15.1")  {
   utils::globalVariables(c("KEYWORD", "FIELD", "PRIM_ID", "Freq", "kwic",
   "primdup", "fixed", "ID", "IDKW", "TYPE", "SET_NO", "ind", "V1", "COUNT",
   "DEL", "SPLIT", "K1_PRIM_ID", "K2_PRIM_ID", "Seq", "K", "TEMP", "Ndup", ".",
-   "KW1", "COMBINED"))
+   "KW1", "COMBINED", "iter", "exep", "FuzzydupIDKW", "FuzzydupIDKW2",
+  "FuzzydupID", "PhoneticdupIDKW", "PhoneticdupID", "SemanticdupIDKW",
+  "SemanticdupID"))
 }
\ No newline at end of file
